@@ -8,7 +8,7 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
-# Copyright (c) 2003 Kenneth J. Pronovici.
+# Copyright (c) 2003-2005 Kenneth J. Pronovici.
 # All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
@@ -25,17 +25,16 @@
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
-# Language : Python (>= 2.2)
+# Language : Python (>= 2.3)
 # Project  : WordUtils
 # Revision : $Id: dawg.py,v 1.35 2003/07/16 05:32:19 pronovic Exp $
 # Purpose  : Implementation of directed acyclic word graph (dawg).
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
-# This file was created with a width of 132 characters, and NO tabs.
 
-######################
-# Pydoc documentation
-######################
+########################################################################
+# Module documentation
+########################################################################
 
 """
 Implementation of a DAWG and associated trie.
@@ -151,21 +150,11 @@ Documentation Notes
    arguments to the private methods should be fairly obvious given the
    documentation for the public methods.  
 
-   I use U{Epydoc<http://epydoc.sourceforge.net>} for source code documentation.
-   I think that it generally does a better job than the alternatives, and I
-   think that the markup is generally very readable.  There's a problem, though.
-   Epydoc doesn't seem to know what to do with static methods.  Because of the
-   way static methods are declared in Python code, Epydoc seems to get them
-   confused with global variables.  The good news is, the only static methods
-   used below are on the Node object, any no one should need to use a Node
-   directly anyway.
-
    Keep in mind that any backslash characters in Epydoc markup must be escaped,
    so in the markup you'll always see two of them together.  This is sometimes
    a bit confusing.
 
 @author: Kenneth J. Pronovici <pronovic@ieee.org>
-@version: $Revision: 1.35 $
 """
 
 
@@ -174,14 +163,9 @@ Documentation Notes
 ########################################################################
 
 import sys
-try:
-   import os
-   import marshal
-   import shutil
-except ImportError, e:
-    print "Failed to import standard modules: %s" % e
-    print "Please try setting the PYTHONPATH environment variable."
-    sys.exit(1)
+import os
+import marshal
+import shutil
 
 
 #######################################################################
@@ -214,28 +198,20 @@ class DawgError(Exception):
 # Node class
 #############
 
-class Node:
+class Node(object):
 
    ######################
    # Class documentation
    ######################
 
-   """Class representing a node in an in-memory trie or dawg."""
+   """
+   Class representing a node in an in-memory trie or dawg.
+   """
 
 
-   #####################
-   # Instance variables
-   #####################
-
-   # These are "permanent" pieces of information
-   value = None
-   endpoint = False
-   children = { }
-
-   
-   ####################
-   # __init__() method
-   ####################
+   ##############
+   # Constructor
+   ##############
 
    def __init__(self, key=None):
       """
@@ -243,12 +219,62 @@ class Node:
       @param key: key to be used to construct this node
       @type key: single-character string 
       """
-      if key is not None:
-         self.value = key[0].lower()
+      self._value = None
+      self._endpoint = False
+      self._children = { }
+      if key is not None: 
+         self._value = key[0].lower()
       else:
-         self.value = None
-      self.endpoint = False
-      self.children = { }
+         self._value = None
+
+
+   #############
+   # Properties
+   #############
+
+   def _setValue(self, value):
+      """
+      Property target used to set value.
+      """
+      self._value = value
+
+   def _getValue(self):
+      """
+      Property target used to get value.
+      """
+      return self._value
+
+   def _setEndpoint(self, value):
+      """
+      Property target used to set endpoint.
+      We normalize the value to C{True} or C{False}.
+      """
+      if value:
+         self._endpoint = True
+      else:
+         self._endpoint = False
+
+   def _getEndpoint(self):
+      """
+      Property target used to get endpoint.
+      """
+      return self._endpoint
+
+   def _setChildren(self, value):
+      """
+      Property target used to set children.
+      """
+      self._children = value
+
+   def _getChildren(self):
+      """
+      Property target used to get children.
+      """
+      return self._children
+
+   value = property(_getValue, _setValue, None, "Value of the node.")
+   endpoint = property(_getEndpoint, _setEndpoint, None, "Indicates whether node is a word endpoint.")
+   children = property(_getChildren, _setChildren, None, "Children of this node.")
 
 
    ####################
@@ -361,7 +387,7 @@ class Node:
       Note: this is a static method.
 
       @param fromnode: node to merge from
-      @type fronnode: L{Node} instance
+      @type fromnode: L{Node} instance
 
       @param tonode: node to merge to
       @type tonode: L{Node} instance
@@ -412,31 +438,21 @@ class Node:
 # Dawg class
 #############
 
-class Dawg:
+class Dawg(object):
 
 
    ######################
    # Class documentation
    ######################
 
-   """Class representing a Directed Acyclic Word Graph, or DAWG."""
+   """
+   Class representing a Directed Acyclic Word Graph, or DAWG.
+   """
 
 
-   #####################
-   # Instance variables
-   #####################
-
-   # These are "permanent" pieces of information
-   root = None        # root node of trie/dawg
-   type = None        # IN_MEMORY or ON_DISK
-   db = None          # location of database on disk
-   wl = None          # location of wordlist on disk
-   indexed = False    # indicates whether the dawg has been indexed
-
-
-   ####################
-   # __init__() method
-   ####################
+   ##############
+   # Constructor
+   ##############
 
    def __init__(self, type=IN_MEMORY, db=None, wl=None):
       """
@@ -456,12 +472,68 @@ class Dawg:
 
       @raise DawgError: under exception circumstances
       """
+
       if type not in [IN_MEMORY, ON_DISK]:
-         raise DawgError, "Dawg type must be either IN_MEMORY or ON_DISK."
-      self.type = type
+         raise DawgError("Dawg type must be either IN_MEMORY or ON_DISK.")
+      self._root = None
+      self._type = type
+      self._db = None
+      self._wl = None
+      self._indexed = False
       self.loadFile(db, wl)
 
 
+   #############
+   # Properties
+   #############
+
+   def _getRoot(self):
+      """
+      Property target used to get root.
+      """
+      return self._root
+
+   def _getType(self):
+      """
+      Property target used to get type.
+      """
+      return self._type
+
+   def _getDb(self):
+      """
+      Property target used to get db.
+      """
+      return self._db
+
+   def _getWl(self):
+      """
+      Property target used to get wl.
+      """
+      return self._wl
+
+   def _setIndexed(self, value):
+      """
+      Property target used to set indexed.
+      We normalize the value to C{True} or C{False}.
+      """
+      if value:
+         self._indexed = True
+      else:
+         self._indexed = False
+
+   def _getIndexed(self):
+      """
+      Property target used to get indexed.
+      """
+      return self._indexed
+
+   root = property(_getRoot, None, None, "Root node of trie/dawg.")
+   type = property(_getType, None, None, "Dawg type (IN_MEMORY or ON_DISK).")
+   db = property(_getDb, None, None, "Location of database on disk.")
+   wl = property(_getWl, None, None, "Location of wordlist on disk.")
+   indexed = property(_getIndexed, None, None, "Indicates whether the dawg has been indexed.")
+
+
    ####################
    # loadFile() method
    ####################
@@ -491,35 +563,35 @@ class Dawg:
       @raise DawgError: under exception circumstances
       """
       if db is not None and wl is not None:
-         raise DawgError, "Object may be initialized with either database or wordlist, not both."
+         raise DawgError("Object may be initialized with either database or wordlist, not both.")
       elif db is not None or wl is not None:
-         oldroot = self.root
-         olddb = self.db
-         oldwl = self.wl
-         self.root = None
-         self.db = None
-         self.wl = None
+         oldroot = self._root
+         olddb = self._db
+         oldwl = self._wl
+         self._root = None
+         self._db = None
+         self._wl = None
          try:
-            if self.type == IN_MEMORY:
+            if self._type == IN_MEMORY:
                if db is not None:
-                  self.db = db
-                  self.root = self._mem_loadFileDatabase(self.db)
+                  self._db = db
+                  self._root = self._mem_loadFileDatabase(self._db)
                else:
-                  self.wl = wl
-                  self.root = self._mem_loadFileWordlist(self.wl)
-            elif self.type == ON_DISK:
+                  self._wl = wl
+                  self._root = self._mem_loadFileWordlist(self._wl)
+            elif self._type == ON_DISK:
                if db is not None:
                   if not os.path.exists(db):
-                     raise IOError
-                  self.db = db
+                     raise IOError("Database [%s] does not exist on disk." % db)
+                  self._db = db
                else:
-                  raise DawgError, "On-disk dawg must be loaded from database."
+                  raise DawgError("On-disk dawg must be loaded from database.")
          except Exception, detail:
             # If this process fails, make sure to reset the state
-            self.root = oldroot
-            self.db = olddb
-            self.wl = oldwl
-            raise DawgError, "%s" % detail
+            self._root = oldroot
+            self._db = olddb
+            self._wl = oldwl
+            raise DawgError("%s" % detail)
 
    def _mem_loadFileWordlist(self, wl):
       root = None
@@ -528,35 +600,35 @@ class Dawg:
       return root
 
    def _mem_loadFileDatabase(self, db):
-      file = open(db, "r+b")
+      f = open(db, "r+b")
       try:
-         root = marshal.load(file)
-         return self._mem_loadFileDatabase_r(file, root)
+         root = marshal.load(f)
+         return self._mem_loadFileDatabase_r(f, root)
       except ValueError, detail:
-         raise DawgError, "Unable to load (%s): %s - maybe not a database?" % (db, detail)
+         raise DawgError("Unable to load (%s): %s - maybe not a database?" % (db, detail))
 
-   def _mem_loadFileDatabase_r(self, file, fp):
+   def _mem_loadFileDatabase_r(self, f, fp):
       node = None
       if fp != -1:
          node = Node()
-         file.seek(fp)
-         value = marshal.load(file)
+         f.seek(fp)
+         value = marshal.load(f)
          if value == -1:
             node.value = None
          else:
             node.value = chr(value)
-         endpoint = marshal.load(file)
+         endpoint = marshal.load(f)
          if endpoint == 0:
             node.endpoint = False
          else:
             node.endpoint = True
-         count = marshal.load(file)
+         count = marshal.load(f)
          childfps = []
          for i in range(0, count):
-            childfps.append(marshal.load(file))
+            childfps.append(marshal.load(f))
          node.children = { }
          for childfp in childfps:
-            child = self._mem_loadFileDatabase_r(file, childfp)
+            child = self._mem_loadFileDatabase_r(f, childfp)
             node.children[child] = child
       return node
 
@@ -582,23 +654,23 @@ class Dawg:
 
       @raise DawgError: under exception circumstances
       """
-      oldroot = self.root
-      olddb = self.db
-      oldwl = self.wl
-      self.root = None
-      self.db = None
-      self.wl = None
+      oldroot = self._root
+      olddb = self._db
+      oldwl = self._wl
+      self._root = None
+      self._db = None
+      self._wl = None
       try:
-         if self.type == IN_MEMORY:
-            self.root = self._mem_loadList(list)
-         elif self.type == ON_DISK:
-            raise DawgError, "On-disk dawg must be loaded from database."
+         if self._type == IN_MEMORY:
+            self._root = self._mem_loadList(list)
+         elif self._type == ON_DISK:
+            raise DawgError("On-disk dawg must be loaded from database.")
       except Exception, detail:
          # If this process fails, make sure to reset the state
-         self.root = oldroot
-         self.db = olddb
-         self.wl = oldwl
-         raise DawgError, "%s" % detail
+         self._root = oldroot
+         self._db = olddb
+         self._wl = oldwl
+         raise DawgError("%s" % detail)
 
    def _mem_loadList(self, list):
       root = None
@@ -646,10 +718,10 @@ class Dawg:
 
       @raise DawgError: under exception circumstances
       """
-      if self.type == ON_DISK:
-         raise DawgError, "Clone functionality is not yet implemented for on-disk dawgs."
+      if self._type == ON_DISK:
+         raise DawgError("Clone functionality is not yet implemented for on-disk dawgs.")
       clone = Dawg()
-      clone.root = self._mem_clone(node)
+      clone._root = self._mem_clone(node)
       return clone
 
    def _mem_clone(self, node):
@@ -683,24 +755,24 @@ class Dawg:
 
       @raise DawgError: under exception circumstances
       """
-      if self.type == ON_DISK:
-         raise DawgError, "Size functionality is not yet implemented for on-disk dawgs."
-      dict = { }
-      self._mem_size(self.root, dict)
-      unique = len(dict.keys())
+      if self._type == ON_DISK:
+         raise DawgError("Size functionality is not yet implemented for on-disk dawgs.")
+      d = { }
+      self._mem_size(self._root, d)
+      unique = len(d.keys())
       references = 0
-      for key in dict.keys():
-         references += dict[key]
+      for key in d.keys():
+         references += d[key]
       return(unique, references)
 
-   def _mem_size(self, node, dict):
+   def _mem_size(self, node, d):
       if node is not None:
          try:
-            dict[node] += 1
+            d[node] += 1
          except:
-            dict[node] = 1
+            d[node] = 1
          for child in node.children.keys():
-            self._mem_size(child, dict)
+            self._mem_size(child, d)
 
 
    #######################
@@ -722,14 +794,14 @@ class Dawg:
 
       @raise DawgError: under exception circumstances
       """
-      if self.type == ON_DISK:
-         raise DawgError, "Indexing functionality is not yet implemented for on-disk dawgs."
+      if self._type == ON_DISK:
+         raise DawgError("Indexing functionality is not yet implemented for on-disk dawgs.")
       self.clearIndex()
-      self.depths = { }
-      self._mem_buildIndex(self.root, None, self.depths, 0)
-      self.indexed = True
+      self._depths = { }
+      self._mem_buildIndex(self._root, None, self._depths, 0)
+      self._indexed = True
 
-   def _mem_buildIndex(self, node, parent, dict, depth):
+   def _mem_buildIndex(self, node, parent, d, depth):
       if node is not None:
          if parent is None:
             node.parents = { }
@@ -742,11 +814,11 @@ class Dawg:
          node.depth = depth
          node.orphan = False
          try:
-            dict[depth].append(node)
+            d[depth].append(node)
          except:
-            dict[depth] = [ node ]
+            d[depth] = [ node ]
          for child in node.children.keys():
-            self._mem_buildIndex(child, node, dict, depth+1)
+            self._mem_buildIndex(child, node, d, depth+1)
 
 
    ######################
@@ -761,13 +833,13 @@ class Dawg:
 
       @raise DawgError: under exception circumstances
       """
-      if self.type == ON_DISK:
-         raise DawgError, "Indexing functionality is not yet implemented for on-disk dawgs."
+      if self._type == ON_DISK:
+         raise DawgError("Indexing functionality is not yet implemented for on-disk dawgs.")
       try:
-         del self.depths
+         del self._depths
       except: pass
-      self._mem_clearIndex(self.root)
-      self.indexed = False
+      self._mem_clearIndex(self._root)
+      self._indexed = False
 
    def _mem_clearIndex(self, node):
       if node is not None:
@@ -798,21 +870,21 @@ class Dawg:
 
       When indexing is complete, the first pass at at compression can begin.
       The main purpose of the first pass is to merge identical leaves.  We
-      build a list of leaves, and merge any leaves that have the same value
-      (all leaves are by definition endpoints, and can have no children, so
-      there's no need to do any comparisons against anything other than value).
+      build a list of leaves, and merge any leaves that have the same value.
+      All leaves are by definition endpoints, and can have no children, so
+      there's no need to do any comparisons against anything other than value.
 
       After all of the leaves have been merged, we move onto the second pass at
       compression.  This pass always looks one "level" up from the current
       level.  For instance, the first time through, it looks at the parents of
-      the leaves.  We'll compare each of the node, and if any two are identical
+      the leaves.  We'll compare each of the nodes, and if any two are identical
       [1], we'll merge them.  When this process is complete, we'll construct a
       new list, up one level (i.e.  parents of the nodes we just operated on,
       which the first time through would be the parents of the parents of the
       leaves) and start all over again.  The whole thing continues until the
       new list of parents contains just one node, the top node.  There is no
       other valid exit condition, since the trie does not necessarily have a
-      constant depth (our input words may vary in length).
+      constant depth (because our input words may vary in length).
 
       [1] It is important to note that for the second pass, two nodes can only be
       considered identical if they have the same value, the same endpoint flag
@@ -854,24 +926,26 @@ class Dawg:
       @type ratio: boolean C{True}/C{False}
 
       @return: 
-            - compression relative to original, as a percentage,
-              if C{ratio} is True
-            - C{None} if C{ratio} is False
+         - compression relative to original, as a percentage, if C{ratio} is True
+         - C{None} if C{ratio} is False
 
       @raise DawgError: under exception circumstances
       """
-      if self.type == ON_DISK:
-         raise DawgError, "On-disk dawgs may not be compressed. To compress, create in-memory database from db file."
+      if self._type == ON_DISK:
+         raise DawgError("On-disk dawgs may not be compressed. To compress, create in-memory database from db file.")
       if ratio:
-         (original,references) = self.size()
+         (original, references) = self.size()
       self._mem_compress()
       if ratio:
-         (compressed,references) = self.size()
-         return 100.0 - ( (float(compressed)/float(original)) * 100.0)
+         (compressed, references) = self.size()
+         if original > 0:
+            return 100.0 - ((float(compressed)//float(original)) * 100.0)
+         else:
+            return 0
       return None
 
    def _mem_compress(self):
-      if not self.indexed:
+      if not self._indexed:
          self.buildIndex()
       leaves = self._mem_mergeLeaves()
       self._mem_mergeNonLeaves(leaves)
@@ -932,20 +1006,20 @@ class Dawg:
       @return: list of leaves at L{Node} instances
       @raise DawgError: under exception circumstances
       """
-      if self.type == ON_DISK:
-         raise DawgError, "The leaves() method has not yet been implemented for on-disk dawgs."
-      dict = { }
-      self._mem_leaves(self.root, dict)
-      return dict.keys()
+      if self._type == ON_DISK:
+         raise DawgError("The leaves() method has not yet been implemented for on-disk dawgs.")
+      d = { }
+      self._mem_leaves(self._root, d)
+      return d.keys()
 
-   def _mem_leaves(self, node, dict):
+   def _mem_leaves(self, node, d):
       if node is not None:
          keys = node.children.keys()
          if len(keys) == 0:
-            dict[node] = node
+            d[node] = node
          else:
             for child in keys:
-               self._mem_leaves(child, dict)
+               self._mem_leaves(child, d)
 
 
    #################
@@ -974,13 +1048,13 @@ class Dawg:
 
       @raise DawgError: under exception circumstances
       """
-      if self.type == ON_DISK:
-         raise DawgError, "The level() method has not yet been implemented for on-disk dawgs."
-      list = self._mem_level(level)
-      return list
+      if self._type == ON_DISK:
+         raise DawgError("The level() method has not yet been implemented for on-disk dawgs.")
+      l = self._mem_level(level)
+      return l
 
    def _mem_level(self, level):
-      if not self.indexed:
+      if not self._indexed:
          self.buildIndex()
       nodes = self.leaves()
       for i in range(0, level):
@@ -1015,10 +1089,10 @@ class Dawg:
       """
       if func is None:
          func = lambda x: sys.stdout.write("%s\n" % x)
-      if self.type == IN_MEMORY:
-         self._mem_traverse(self.root, "", func)
+      if self._type == IN_MEMORY:
+         self._mem_traverse(self._root, "", func)
       else:
-         self._disk_traverse(self.db, "", func)
+         self._disk_traverse(self._db, "", func)
 
    def _mem_traverse(self, node, word, func):
       if node is not None:
@@ -1032,28 +1106,28 @@ class Dawg:
    def _disk_traverse(self, filepath, word, func):
       if filepath is not None:
          if os.path.exists(filepath) and os.stat(filepath)[6] > 0:
-            file = open(filepath, "r+b")
-            file.seek(0)
+            f = open(filepath, "r+b")
+            f.seek(0)
             try:
-               root = marshal.load(file)
-               self._disk_traverse_r(file, root, word, func)
+               root = marshal.load(f)
+               self._disk_traverse_r(f, root, word, func)
             except ValueError, detail:
-               raise DawgError, "Unable to load (%s): %s - maybe not a database?" % (filepath, detail)
+               raise DawgError("Unable to load (%s): %s - maybe not a database?" % (filepath, detail))
 
-   def _disk_traverse_r(self, file, fp, word, func):
+   def _disk_traverse_r(self, f, fp, word, func):
       if fp != -1:
-         file.seek(fp)
-         value = marshal.load(file)
+         f.seek(fp)
+         value = marshal.load(f)
          if value != -1:
             word += chr(value)
-         if marshal.load(file) != 0:
+         if marshal.load(f) != 0:
             func(word)
-         count = marshal.load(file)
+         count = marshal.load(f)
          childfps = []
          for i in range(0, count):
-            childfps.append(marshal.load(file))
+            childfps.append(marshal.load(f))
          for childfp in childfps:
-            self._disk_traverse_r(file, childfp, word, func)
+            self._disk_traverse_r(f, childfp, word, func)
 
 
    ################
@@ -1062,7 +1136,7 @@ class Dawg:
 
    def save(self, db=None, wl=None, overwrite=False):
       """
-      Writes a dawg to disk as a wordlist and/or in a portable "database"
+      Writes a dawg to disk as a wordlist and/or a portable "database"
       format.
 
       Both the C{wl} and C{db} arguments may be provided.  This will
@@ -1085,13 +1159,12 @@ class Dawg:
       if wl is not None:
          self._saveWordlist(wl, overwrite)
       if db is not None:
-         self._saveDatabase(self.root, self.db, db, overwrite)
+         self._saveDatabase(self._root, self._db, db, overwrite)
       
    def _saveWordlist(self, newwl, overwrite):
       if os.path.exists(newwl) and not overwrite:
-         raise DawgError, "Wordlist file (%s) exists but overwrite not specified." % newwl
-      list = self.list()
-      if len(list) == 0:
+         raise DawgError("Wordlist file (%s) exists but overwrite not specified." % newwl)
+      if len(self.list()) == 0:
          open(newwl, "w").write("")
       else:
          fp = open(newwl, "w")
@@ -1100,23 +1173,23 @@ class Dawg:
 
    def _saveDatabase(self, root, db, newdb, overwrite):
       if db == newdb:
-         raise DawgError, "Object's current database (%s) may not be overwritten by save." % db
+         raise DawgError("Object's current database (%s) may not be overwritten by save." % db)
       if os.path.exists(newdb) and not overwrite:
-         raise DawgError, "Database file (%s) exists but overwrite not specified." % newdb
-      if self.type == IN_MEMORY:
+         raise DawgError("Database file (%s) exists but overwrite not specified." % newdb)
+      if self._type == IN_MEMORY:
          self._mem_saveDatabase(root, newdb)
       else:
          self._disk_saveDatabase(db, newdb)
 
    def _mem_saveDatabase(self, node, newdb):
       fps = { }
-      file = open(newdb, "w+b")
-      marshal.dump(int(0), file)
-      root = self._mem_saveDatabase_r(node, file, fps)
-      file.seek(0)
-      marshal.dump(int(root), file)
+      f = open(newdb, "w+b")
+      marshal.dump(int(0), f)
+      root = self._mem_saveDatabase_r(node, f, fps)
+      f.seek(0)
+      marshal.dump(int(root), f)
 
-   def _mem_saveDatabase_r(self, node, file, fps):
+   def _mem_saveDatabase_r(self, node, f, fps):
       node_fp = -1
       if node is not None:
          try:
@@ -1124,17 +1197,17 @@ class Dawg:
          except KeyError:
             childfps = []
             for child in node.children.keys():
-               childfps.append(self._mem_saveDatabase_r(child, file, fps))
-            node_fp = file.tell()
+               childfps.append(self._mem_saveDatabase_r(child, f, fps))
+            node_fp = f.tell()
             fps[node] = node_fp
             if node.value is None:
-               marshal.dump(int(-1), file)
+               marshal.dump(int(-1), f)
             else:
-               marshal.dump(int(ord(node.value)), file)
-            marshal.dump(int(node.endpoint), file)
-            marshal.dump(int(len(childfps)), file)
+               marshal.dump(int(ord(node.value)), f)
+            marshal.dump(int(node.endpoint), f)
+            marshal.dump(int(len(childfps)), f)
             for childfp in childfps:
-               marshal.dump(int(childfp), file)
+               marshal.dump(int(childfp), f)
       return node_fp
 
    def _disk_saveDatabase(self, db, newdb):
@@ -1159,10 +1232,10 @@ class Dawg:
 
       @raise DawgError: under exception circumstances
       """
-      if self.type == IN_MEMORY:
-         return self._mem_search(self.root, key.lower())
+      if self._type == IN_MEMORY:
+         return self._mem_search(self._root, key.lower())
       else:
-         return self._disk_search(self.db, key.lower())
+         return self._disk_search(self._db, key.lower())
    
    def _mem_search(self, node, key):
       if node is not None:
@@ -1175,37 +1248,37 @@ class Dawg:
    def _disk_search(self, filepath, key):
       if filepath is not None:
          if os.path.exists(filepath) and os.stat(filepath)[6] > 0:
-            file = open(filepath, "r+b")
-            file.seek(0)
+            f = open(filepath, "r+b")
+            f.seek(0)
             try:
-               root = marshal.load(file)
-               return self._disk_search_r(file, root, key)
+               root = marshal.load(f)
+               return self._disk_search_r(f, root, key)
             except ValueError, detail:
-               raise DawgError, "Unable to load (%s): %s - maybe not a database?" % (filepath, detail)
+               raise DawgError("Unable to load (%s): %s - maybe not a database?" % (filepath, detail))
 
-   def _disk_search_r(self, file, fp, key):
+   def _disk_search_r(self, f, fp, key):
       if fp != -1:
-         file.seek(fp)
-         marshal.load(file)      # value
-         marshal.load(file)      # endpoint
-         count = marshal.load(file)
+         f.seek(fp)
+         marshal.load(f)      # value
+         marshal.load(f)      # endpoint
+         count = marshal.load(f)
          childfps = []
          for i in range(0, count):
-            childfps.append(marshal.load(file))
+            childfps.append(marshal.load(f))
          if len(key) == 1:
             for childfp in childfps:
-               file.seek(childfp)
-               value = marshal.load(file)
-               endpoint = marshal.load(file) 
+               f.seek(childfp)
+               value = marshal.load(f)
+               endpoint = marshal.load(f) 
                if value != -1 and chr(value) == key[0]:
                   if endpoint != 0:
                      return True
             return False
          for childfp in childfps:
-            file.seek(childfp)
-            value = marshal.load(file)
+            f.seek(childfp)
+            value = marshal.load(f)
             if value != -1 and chr(value) == key[0]:
-               return self._disk_search_r(file, childfp, key[1:])
+               return self._disk_search_r(f, childfp, key[1:])
       return False
 
 
@@ -1225,10 +1298,10 @@ class Dawg:
 
       @raise DawgError: under exception circumstances
       """
-      if self.type == IN_MEMORY:
-         results = self._mem_patternSearch(self.root, pattern, "")
+      if self._type == IN_MEMORY:
+         results = self._mem_patternSearch(self._root, pattern, "")
       else:
-         results = self._disk_patternSearch(self.db, pattern, "")
+         results = self._disk_patternSearch(self._db, pattern, "")
       results.sort()
       return results
       
@@ -1257,52 +1330,52 @@ class Dawg:
    def _disk_patternSearch(self, filepath, pattern, word):
       if filepath is not None:
          if os.path.exists(filepath) and os.stat(filepath)[6] > 0:
-            file = open(filepath, "r+b")
-            file.seek(0)
+            f = open(filepath, "r+b")
+            f.seek(0)
             try:
-               root = marshal.load(file)
-               return self._disk_patternSearch_r(file, root, pattern, word)
+               root = marshal.load(f)
+               return self._disk_patternSearch_r(f, root, pattern, word)
             except ValueError, detail:
-               raise DawgError, "Unable to load (%s): %s - maybe not a database?" % (filepath, detail)
+               raise DawgError("Unable to load (%s): %s - maybe not a database?" % (filepath, detail))
 
-   def _disk_patternSearch_r(self, file, fp, pattern, word):
+   def _disk_patternSearch_r(self, f, fp, pattern, word):
       results = []
       if fp != -1:
-         file.seek(fp)
-         value = marshal.load(file)
-         endpoint = marshal.load(file)
-         count = marshal.load(file)
+         f.seek(fp)
+         value = marshal.load(f)
+         endpoint = marshal.load(f)
+         count = marshal.load(f)
          childfps = []
          for i in range(0, count):
-            childfps.append(marshal.load(file))
+            childfps.append(marshal.load(f))
          if value != -1:
             word += chr(value)
          if pattern[0] == ".":
             if len(pattern) == 1:
                for childfp in childfps:
-                  file.seek(childfp)
-                  value = marshal.load(file)
-                  endpoint = marshal.load(file)
+                  f.seek(childfp)
+                  value = marshal.load(f)
+                  endpoint = marshal.load(f)
                   if endpoint != 0:
                      results.append("%s%s" % (word, chr(value)))
                return results
             for childfp in childfps:
-               results += self._disk_patternSearch_r(file, childfp, pattern[1:], word)
+               results += self._disk_patternSearch_r(f, childfp, pattern[1:], word)
          else:
             if len(pattern) == 1:
                for childfp in childfps:
-                  file.seek(childfp)
-                  value = marshal.load(file)
-                  endpoint = marshal.load(file) 
+                  f.seek(childfp)
+                  value = marshal.load(f)
+                  endpoint = marshal.load(f) 
                   if value != -1 and chr(value) == pattern[0]:
                      if endpoint != 0:
                         results.append("%s%s" % (word, chr(value)))
                return results
          for childfp in childfps:
-            file.seek(childfp)
-            value = marshal.load(file)
+            f.seek(childfp)
+            value = marshal.load(f)
             if value != -1 and chr(value) == pattern[0]:
-               results += self._disk_patternSearch_r(file, childfp, pattern[1:], word)
+               results += self._disk_patternSearch_r(f, childfp, pattern[1:], word)
       return results
       
       
@@ -1333,12 +1406,12 @@ class Dawg:
 
       @raise DawgError: under exception circumstances
       """
-      if self.type == ON_DISK:
-         raise DawgError, "On-disk dawgs read-only.  To modify, create in-memory database from db file."
-      list = self.list()
-      list.append(key)
-      list.sort()
-      self.loadList(list)
+      if self._type == ON_DISK:
+         raise DawgError("On-disk dawgs read-only.  To modify, create in-memory database from db file.")
+      l = self.list()
+      l.append(key)
+      l.sort()
+      self.loadList(l)
 
 
    ##################
@@ -1367,11 +1440,11 @@ class Dawg:
 
       @raise DawgError: under exception circumstances
       """
-      if self.type == ON_DISK:
-         raise DawgError, "On-disk dawgs are read-only.  To modify, create in-memory database from db file."
-      list = self.list()
-      list.remove(key)
-      self.loadList(list)
+      if self._type == ON_DISK:
+         raise DawgError("On-disk dawgs are read-only.  To modify, create in-memory database from db file.")
+      l = self.list()
+      l.remove(key)
+      self.loadList(l)
 
 
    ###################
@@ -1379,8 +1452,8 @@ class Dawg:
    ###################
 
    def _insert(self, root, key):
-      if self.type == ON_DISK:
-         raise DawgError, "On-disk dawgs are read-only.  To modify, create in-memory database from db file."
+      if self._type == ON_DISK:
+         raise DawgError("On-disk dawgs are read-only.  To modify, create in-memory database from db file.")
       if root is None:
          root = Node()
       self._mem_insert(root, key)
@@ -1393,12 +1466,3 @@ class Dawg:
       else:
          self._mem_insert(child, key[1:])
 
-
-########################################################################
-# Module entry point
-########################################################################
-
-# Ensures that the module isn't run if someone just imports it.
-if __name__ == '__main__':
-   print "Module just exists to be imported, sorry."
-
