@@ -9,32 +9,32 @@
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
-# Language : Python (>= 2.2)
+# Language : Python (>= 2.3)
 # Project  : WordUtils
 # Revision : $Id: release.py,v 1.1 2003/07/16 05:23:04 pronovic Exp $
 # Purpose  : Provides location to maintain version information.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
-# This file was created with a width of 132 characters, and NO tabs.
-
-
-########
-# Notes
-########
 
 """
 Provides location to maintain version information.
 
+@sort: AUTHOR, EMAIL, COPYRIGHT, VERSION, DATE, URL
+
+@var AUTHOR: Author of software.
+@var EMAIL: Email address of author.
+@var COPYRIGHT: Copyright date.
+@var VERSION: Software version.
+@var DATE: Software release date.
+@var URL: URL of WordUtils webpage.
+
 @author: Kenneth J. Pronovici <pronovic@ieee.org>
-@version: $Revision: 1.1 $
 """
 
-
-########################################################################
-# Version information
-########################################################################
-
-VERSION     = "0.7.2"
-DATE        = "09 Sep 2003"
-
+AUTHOR      = "Kenneth J. Pronovici"
+EMAIL       = "pronovic@ieee.org"
+COPYRIGHT   = "2003-2005"
+VERSION     = "0.8.0"
+DATE        = "27 Sep 2005"
+URL         = "http://www.cedar-solutions.com/software/wordutils/"
 
