@@ -8,7 +8,7 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
-# Copyright (c) 2003 Kenneth J. Pronovici.
+# Copyright (c) 2003-2005 Kenneth J. Pronovici.
 # All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
@@ -25,17 +25,16 @@
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
-# Language : Python (>= 2.2)
+# Language : Python (>= 2.3)
 # Project  : WordUtils
 # Revision : $Id: tree.py,v 1.18 2003/07/16 05:23:04 pronovic Exp $
 # Purpose  : Implementation of a ternary search tree
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
-# This file was created with a width of 132 characters, and NO tabs.
 
-######################
-# Pydoc documentation
-######################
+########################################################################
+# Module documentation
+########################################################################
 
 """
 Implementation of a ternary search tree and related functionality.
@@ -138,7 +137,6 @@ Documentation Notes
    a bit confusing. 
 
 @author: Kenneth J. Pronovici <pronovic@ieee.org>
-@version: $Revision: 1.18 $
 """
 
 
@@ -147,14 +145,9 @@ Documentation Notes
 ########################################################################
 
 import sys
-try:
-   import os
-   import shutil
-   import marshal
-except ImportError, e:
-    print "Failed to import standard modules: %s" % e
-    print "Please try setting the PYTHONPATH environment variable."
-    sys.exit(1)
+import os
+import shutil
+import marshal
 
 
 #######################################################################
@@ -187,73 +180,116 @@ class TreeError(Exception):
 # Node class
 #############
 
-class Node:
-
-   ######################
-   # Class documentation
-   ######################
+class Node(object):
 
    """Class representing a node in an in-memory ternary search tree."""
 
-
-   #####################
-   # Instance variables
-   #####################
-
-   splitchar = None
-   endpoint = False
-   lokid = None
-   eqkid = None
-   hikid = None
-   
-   
-   ####################
-   # __init__() method
-   ####################
-
    def __init__ (self, key=None):
       """
       Default constructor.
       @param key: key to be used to construct this node
       @type key: single-character string     
       """
+      self._splitchar = None
+      self._endpoint = False
+      self._lokid = None
+      self._eqkid = None
+      self._hikid = None
       if key is not None:
          self.splitchar = key[0].lower()
       else:
          self.splitchar = None
-      self.endpoint = False
-      self.lokid = None
-      self.eqkid = None
-      self.hikid = None
+
+   def _setSplitchar(self, value):
+      """
+      Property target used to set splitchar.
+      """
+      self._splitchar = value
+
+   def _getSplitchar(self):
+      """
+      Property target used to get splitchar.
+      """
+      return self._splitchar
+
+   def _setEndpoint(self, value):
+      """
+      Property target used to set endpoint.
+      We normalize the value to C{True} or C{False}.
+      """
+      if value:
+         self._endpoint = True
+      else:
+         self._endpoint = False
+
+   def _getEndpoint(self):
+      """
+      Property target used to get endpoint.
+      """
+      return self._endpoint
+
+   def _setLokid(self, value):
+      """
+      Property target used to set lokid.
+      """
+      self._lokid = value
+
+   def _getLokid(self):
+      """
+      Property target used to get lokid.
+      """
+      return self._lokid
+
+   def _setEqkid(self, value):
+      """
+      Property target used to set eqkid.
+      """
+      self._eqkid = value
+
+   def _getEqkid(self):
+      """
+      Property target used to get eqkid.
+      """
+      return self._eqkid
+
+   def _setHikid(self, value):
+      """
+      Property target used to set hikid.
+      """
+      self._hikid = value
+
+   def _getHikid(self):
+      """
+      Property target used to get hikid.
+      """
+      return self._hikid
+
+   splitchar = property(_getSplitchar, _setSplitchar, None, "Split character.")
+   endpoint = property(_getEndpoint, _setEndpoint, None, "Endpoint flag.")
+   lokid = property(_getLokid, _setLokid, None, "Low child.")
+   eqkid = property(_getEqkid, _setEqkid, None, "Equals child.")
+   hikid = property(_getHikid, _setHikid, None, "High child.")
 
 
 ##########################
 # TernarySearchTree class
 ##########################
 
-class TernarySearchTree:
+class TernarySearchTree(object):
 
 
    ######################
    # Class documentation
    ######################
 
-   """Class representing a ternary search tree."""
+   """
+   Class representing a ternary search tree.
+   """
 
 
-   #####################
-   # Instance variables
-   #####################
-
-   root = None        # root node of tree
-   type = None        # IN_MEMORY or ON_DISK
-   db = None          # location of database on disk
-   wl = None          # location of wordlist on disk
-
-
-   ####################
-   # __init__() method
-   ####################
+   ##############
+   # Constructor
+   ##############
 
    def __init__(self, type=IN_MEMORY, db=None, wl=None):
       """
@@ -274,11 +310,48 @@ class TernarySearchTree:
       @raise TreeError: under exception circumstances
       """  
       if type not in [IN_MEMORY, ON_DISK]:
-         raise TreeError, "Tree type must be either IN_MEMORY or ON_DISK."
-      self.type = type
+         raise TreeError("Tree type must be either IN_MEMORY or ON_DISK.")
+      self._root = None
+      self._type = type
+      self._db = None
+      self._wl = None
       self.loadFile(db, wl)
 
 
+   #############
+   # Properties
+   #############
+
+   def _getRoot(self):
+      """
+      Property target used to get root.
+      """
+      return self._root
+
+   def _getType(self):
+      """
+      Property target used to get type.
+      """
+      return self._type
+
+   def _getDb(self):
+      """
+      Property target used to get db.
+      """
+      return self._db
+
+   def _getWl(self):
+      """
+      Property target used to get wl.
+      """
+      return self._wl
+
+   root = property(_getRoot, None, None, "Root node of tree.")
+   type = property(_getType, None, None, "Tree type (IN_MEMORY or ON_DISK).")
+   db = property(_getDb, None, None, "Location of database on disk.")
+   wl = property(_getWl, None, None, "Location of wordlist on disk.")
+
+
    ####################
    # loadFile() method
    ####################
@@ -308,35 +381,35 @@ class TernarySearchTree:
       @raise TreeError: under exception circumstances
       """  
       if db is not None and wl is not None:
-         raise TreeError, "Object may be initialized with either database or wordlist, not both."
+         raise TreeError("Object may be initialized with either database or wordlist, not both.")
       elif db is not None or wl is not None:
-         oldroot = self.root
-         olddb = self.db
-         oldwl = self.wl
-         self.root = None
-         self.db = None
-         self.wl = None
+         oldroot = self._root
+         olddb = self._db
+         oldwl = self._wl
+         self._root = None
+         self._db = None
+         self._wl = None
          try:
-            if self.type == IN_MEMORY:
+            if self._type == IN_MEMORY:
                if db is not None:
-                  self.db = db
-                  self.root = self._mem_loadFileDatabase(self.db)
+                  self._db = db
+                  self._root = self._mem_loadFileDatabase(self._db)
                else:
-                  self.wl = wl
-                  self.root = self._mem_loadFileWordlist(self.wl)
-            elif self.type == ON_DISK:
+                  self._wl = wl
+                  self._root = self._mem_loadFileWordlist(self._wl)
+            elif self._type == ON_DISK:
                if db is not None:
                   if not os.path.exists(db):
-                     raise IOError
-                  self.db = db
+                     raise IOError("Database [%s] does not exist on disk." % db)
+                  self._db = db
                else:
-                  raise TreeError, "On-disk tree must be loaded from database."
+                  raise TreeError("On-disk tree must be loaded from database.")
          except Exception, detail:
             # If this process fails, make sure to reset the state
-            self.root = oldroot
-            self.db = olddb
-            self.wl = oldwl
-            raise TreeError, "%s" % detail
+            self._root = oldroot
+            self._db = olddb
+            self._wl = oldwl
+            raise TreeError("%s" % detail)
 
    def _mem_loadFileWordlist(self, wl):
       node = None
@@ -345,12 +418,12 @@ class TernarySearchTree:
       return node
 
    def _mem_loadFileDatabase(self, db):
-      file = open(db, "r+b")
+      f = open(db, "r+b")
       try:
-         head = marshal.load(file)
-         return self._mem_loadFileDatabase_r(None, file, head)
+         head = marshal.load(f)
+         return self._mem_loadFileDatabase_r(None, f, head)
       except ValueError, detail:
-         raise TreeError, "Unable to load (%s): %s - maybe not a database?" % (db, detail)
+         raise TreeError("Unable to load (%s): %s - maybe not a database?" % (db, detail))
 
    def _mem_loadFileDatabase_r(self, node, file, fp):
       if node is None:
@@ -397,23 +470,23 @@ class TernarySearchTree:
 
       @raise TreeError: under exception circumstances
       """    
-      oldroot = self.root
-      olddb = self.db
-      oldwl = self.wl
-      self.root = None
-      self.db = None
-      self.wl = None
+      oldroot = self._root
+      olddb = self._db
+      oldwl = self._wl
+      self._root = None
+      self._db = None
+      self._wl = None
       try:
-         if self.type == IN_MEMORY:
-            self.root = self._mem_loadList(list)
-         elif self.type == ON_DISK:
-            raise TreeError, "On-disk tree must be loaded from database."
+         if self._type == IN_MEMORY:
+            self._root = self._mem_loadList(list)
+         elif self._type == ON_DISK:
+            raise TreeError("On-disk tree must be loaded from database.")
       except Exception, detail:
          # If this process fails, make sure to reset the state
-         self.root = oldroot
-         self.db = olddb
-         self.wl = oldwl
-         raise TreeError, "%s" % detail
+         self._root = oldroot
+         self._db = olddb
+         self._wl = oldwl
+         raise TreeError("%s" % detail)
 
    def _mem_loadList(self, list):
       node = None
@@ -461,8 +534,8 @@ class TernarySearchTree:
 
       @raise TreeError: under exception circumstances 
       """
-      if self.type == ON_DISK:
-         raise TreeError, "On-disk trees may not be compressed. To compress, create in-memory database from db file."
+      if self._type == ON_DISK:
+         raise TreeError("On-disk trees may not be compressed. To compress, create in-memory database from db file.")
       if ratio:
          return 100.0
       else:
@@ -491,10 +564,10 @@ class TernarySearchTree:
       """
       if func is None:
          func = lambda x: sys.stdout.write("%s\n" % x)
-      if self.type == IN_MEMORY:
-         self._mem_traverse(self.root, "", func)
+      if self._type == IN_MEMORY:
+         self._mem_traverse(self._root, "", func)
       else:
-         self._disk_traverse(self.db, "", func)
+         self._disk_traverse(self._db, "", func)
 
    def _mem_traverse(self, node, word, func):
       if node is not None:
@@ -512,13 +585,13 @@ class TernarySearchTree:
    def _disk_traverse(self, filepath, word, func):
       if filepath is not None:
          if os.path.exists(filepath) and os.stat(filepath)[6] > 0:
-            file = open(filepath, "r+b")
-            file.seek(0)
+            f = open(filepath, "r+b")
+            f.seek(0)
             try:
-               head = marshal.load(file)
-               self._disk_traverse_r(file, head, word, func)
+               head = marshal.load(f)
+               self._disk_traverse_r(f, head, word, func)
             except ValueError, detail:
-               raise TreeError, "Unable to load (%s): %s - maybe not a database?" % (filepath, detail)
+               raise TreeError("Unable to load (%s): %s - maybe not a database?" % (filepath, detail))
 
    def _disk_traverse_r(self, file, fp, word, func):
       if fp != -1:
@@ -568,13 +641,13 @@ class TernarySearchTree:
       if wl is not None:
          self._saveWordlist(wl, overwrite)
       if db is not None:
-         self._saveDatabase(self.root, self.db, db, overwrite)
+         self._saveDatabase(self._root, self._db, db, overwrite)
       
    def _saveWordlist(self, newwl, overwrite):
       if os.path.exists(newwl) and not overwrite:
-         raise TreeError, "Wordlist file (%s) exists but overwrite not specified." % newwl
-      list = self.list()
-      if len(list) == 0:
+         raise TreeError("Wordlist file (%s) exists but overwrite not specified." % newwl)
+      l = self.list()
+      if len(l) == 0:
          open(newwl, "w").write("")
       else:
          fp = open(newwl, "w")
@@ -583,42 +656,42 @@ class TernarySearchTree:
 
    def _saveDatabase(self, root, db, newdb, overwrite):
       if db == newdb:
-         raise TreeError, "Object's current database (%s) may not be overwritten by save." % db
+         raise TreeError("Object's current database (%s) may not be overwritten by save." % db)
       if os.path.exists(newdb) and not overwrite:
-         raise TreeError, "Database file (%s) exists but overwrite not specified." % newdb
-      if self.type == IN_MEMORY:
+         raise TreeError("Database file (%s) exists but overwrite not specified." % newdb)
+      if self._type == IN_MEMORY:
          self._mem_saveDatabase(root, newdb)
       else:
          self._disk_saveDatabase(db, newdb)
 
    def _mem_saveDatabase(self, node, newdb):
-      file = open(newdb, "w+b")
-      marshal.dump(int(0), file)
-      head = self._mem_saveDatabase_r(node, file)
-      file.seek(0)
-      marshal.dump(int(head), file)
+      f = open(newdb, "w+b")
+      marshal.dump(int(0), f)
+      head = self._mem_saveDatabase_r(node, f)
+      f.seek(0)
+      marshal.dump(int(head), f)
 
-   def _mem_saveDatabase_r(self, node, file):
+   def _mem_saveDatabase_r(self, node, f):
       nodefp = -1
       if node is not None:
          lofp = -1
          eqfp = -1
          hifp = -1
          if node.lokid is None:
-            lofp = self._mem_saveDatabase_r(node.lokid, file)
+            lofp = self._mem_saveDatabase_r(node.lokid, f)
          if node.eqkid is not None:
-            eqfp = self._mem_saveDatabase_r(node.eqkid, file)
+            eqfp = self._mem_saveDatabase_r(node.eqkid, f)
          if node.hikid is not None:
-            hifp = self._mem_saveDatabase_r(node.hikid, file)
-         nodefp = file.tell()
+            hifp = self._mem_saveDatabase_r(node.hikid, f)
+         nodefp = f.tell()
          if node.splitchar is None:
             marshal.dump(int(-1))
          else:
-            marshal.dump(int(ord(node.splitchar)), file)
-         marshal.dump(int(node.endpoint), file)
-         marshal.dump(int(lofp), file)
-         marshal.dump(int(eqfp), file)
-         marshal.dump(int(hifp), file)
+            marshal.dump(int(ord(node.splitchar)), f)
+         marshal.dump(int(node.endpoint), f)
+         marshal.dump(int(lofp), f)
+         marshal.dump(int(eqfp), f)
+         marshal.dump(int(hifp), f)
       return nodefp
 
    def _disk_saveDatabase(self, db, newdb):
@@ -643,10 +716,10 @@ class TernarySearchTree:
 
       @raise TreeError: under exception circumstances
       """ 
-      if self.type == IN_MEMORY:
-         return self._mem_search(self.root, key.lower())
+      if self._type == IN_MEMORY:
+         return self._mem_search(self._root, key.lower())
       else:
-         return self._disk_search(self.db, key.lower())
+         return self._disk_search(self._db, key.lower())
    
    def _mem_search(self, node, key):
       i = 0
@@ -665,17 +738,17 @@ class TernarySearchTree:
 
    def _disk_search(self, filepath, key):
       i = 0
-      file = open(filepath, "r+b")
-      file.seek(0)
+      f = open(filepath, "r+b")
+      f.seek(0)
       try:
-         fp = marshal.load(file)
+         fp = marshal.load(f)
          while fp != -1 and i < len(key):
-            file.seek(fp)
-            splitchar = chr(marshal.load(file))
-            endpoint = marshal.load(file)
-            lofp = marshal.load(file)
-            eqfp = marshal.load(file)
-            hifp = marshal.load(file)
+            f.seek(fp)
+            splitchar = chr(marshal.load(f))
+            endpoint = marshal.load(f)
+            lofp = marshal.load(f)
+            eqfp = marshal.load(f)
+            hifp = marshal.load(f)
             if key[i] < splitchar:
                fp = lofp
             elif key[i] == splitchar:
@@ -688,7 +761,7 @@ class TernarySearchTree:
                fp = hifp
          return False
       except ValueError, detail:
-         raise TreeError, "Unable to load (%s): %s - maybe not a database?" % (filepath, detail)
+         raise TreeError("Unable to load (%s): %s - maybe not a database?" % (filepath, detail))
 
 
    #########################
@@ -707,10 +780,10 @@ class TernarySearchTree:
 
       @raise TreeError: under exception circumstances
       """
-      if self.type == IN_MEMORY:
-         return self._mem_patternSearch(self.root, pattern, "")
+      if self._type == IN_MEMORY:
+         return self._mem_patternSearch(self._root, pattern, "")
       else:
-         return self._disk_patternSearch(self.db, pattern, "")
+         return self._disk_patternSearch(self._db, pattern, "")
       
    def _mem_patternSearch(self, node, pattern, word):
       results = []
@@ -732,37 +805,37 @@ class TernarySearchTree:
       return results
 
    def _disk_patternSearch(self, filepath, pattern, word):
-      file = open(filepath, "r+b")
-      file.seek(0)
+      f = open(filepath, "r+b")
+      f.seek(0)
       try:
-         fp = marshal.load(file)
-         return self._disk_patternSearch_r(file, fp, pattern, word)
+         fp = marshal.load(f)
+         return self._disk_patternSearch_r(f, fp, pattern, word)
       except ValueError, detail:
-         raise TreeError, "Unable to load (%s): %s - maybe not a database?" % (filepath, detail)
+         raise TreeError("Unable to load (%s): %s - maybe not a database?" % (filepath, detail))
       
-   def _disk_patternSearch_r(self, file, fp, pattern, word):
+   def _disk_patternSearch_r(self, f, fp, pattern, word):
       results = []
       if fp != -1:
-         file.seek(fp)
-         splitchar = chr(marshal.load(file))
-         endpoint = marshal.load(file)
-         lofp = marshal.load(file)
-         eqfp = marshal.load(file)
-         hifp = marshal.load(file)
+         f.seek(fp)
+         splitchar = chr(marshal.load(f))
+         endpoint = marshal.load(f)
+         lofp = marshal.load(f)
+         eqfp = marshal.load(f)
+         hifp = marshal.load(f)
          word += splitchar
          if pattern[0] == "." or pattern[0] < splitchar:
             if lofp != -1:
-               results += self._disk_patternSearch_r(file, lofp, pattern, word[1:])
+               results += self._disk_patternSearch_r(f, lofp, pattern, word[1:])
          if pattern[0] == "." or pattern[0] == splitchar:
             if len(pattern) == 1:
                if endpoint:
                   results.append(word)
             else:
                if eqfp != -1:
-                  results += self._disk_patternSearch_r(file, eqfp, pattern[1:], word)
+                  results += self._disk_patternSearch_r(f, eqfp, pattern[1:], word)
          if pattern[0] == "." or pattern[0] > splitchar:
             if hifp != -1:
-               results += self._disk_patternSearch_r(file, hifp, pattern, word[:-1])
+               results += self._disk_patternSearch_r(f, hifp, pattern, word[:-1])
       return results
 
       
@@ -790,12 +863,12 @@ class TernarySearchTree:
 
       @raise DawgError: under exception circumstances
       """ 
-      if self.type == ON_DISK:
-         raise TreeError, "On-disk trees are read-only.  To modify, create in-memory database from db file."
-      list = self.list()
-      list.append(key)
-      list.sort()
-      self.loadList(list)
+      if self._type == ON_DISK:
+         raise TreeError("On-disk trees are read-only.  To modify, create in-memory database from db file.")
+      l = self.list()
+      l.append(key)
+      l.sort()
+      self.loadList(l)
 
 
    ##################
@@ -821,11 +894,11 @@ class TernarySearchTree:
 
       @raise DawgError: under exception circumstances
       """ 
-      if self.type == ON_DISK:
-         raise TreeError, "On-disk trees are read-only.  To modify, create in-memory database from db file."
-      list = self.list()
-      list.remove(key)
-      self.loadList(list)
+      if self._type == ON_DISK:
+         raise TreeError("On-disk trees are read-only.  To modify, create in-memory database from db file.")
+      l = self.list()
+      l.remove(key)
+      self.loadList(l)
 
 
    ###################
@@ -833,8 +906,8 @@ class TernarySearchTree:
    ###################
 
    def _insert(self, node, key):
-      if self.type == ON_DISK:
-         raise TreeError, "On-disk trees are read-only.  To modify, create in-memory database from db file."
+      if self._type == ON_DISK:
+         raise TreeError("On-disk trees are read-only.  To modify, create in-memory database from db file.")
       return self._mem_insert(node, key)
 
    def _mem_insert(self, node, key):
@@ -851,12 +924,3 @@ class TernarySearchTree:
          node.hikid = self._mem_insert(node.hikid, key)
       return node
 
-
-########################################################################
-# Module entry point
-########################################################################
-
-# Ensures that the module isn't run if someone just imports it.
-if __name__ == '__main__':
-   print "Module just exists to be imported, sorry."
-
